/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.utils;


// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.BridgeException;

import com.ymock.util.Logger;

// for XML RPC, from org.apache.xmlrpc:xmlrpc-client
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.apache.xmlrpc.client.XmlRpcTransportFactory;

/**
 * Adapter between a reader and XML RPC web service.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: RpcAdapter.java 1614 2011-07-10 19:05:43Z guard $
 */
public final class RpcAdapter {

    /**
     * XmlRpcClient to the XML RPC web service.
     */
    private XmlRpcClient client;

    /**
     * XML RPC configuration.
     */
    private final XmlRpcClientConfigImpl config =
        new XmlRpcClientConfigImpl();

    /**
     * Public ctor, for public access to XML RPC.
     * @param url The URL of XML RPC web service
     * @param suffix URL suffix
     * @throws BridgeException If some problem inside
     */
    public RpcAdapter(final String url, final String suffix)
        throws BridgeException {
        this.config.setServerURL(new URLBuilder(url).append(suffix).build());
    }

    /**
     * Set authentication.
     * @param login The login
     * @param password The password
     */
    public void setAuth(final String login, final String password) {
        this.config.setBasicUserName(login);
        this.config.setBasicPassword(password);
    }

    /**
     * Set transport factory by class name.
     * @param name The factory to set (class name)
     * @throws BridgeException If some problem inside
     */
    public void setTransportFactory(final String name) throws BridgeException {
        Object factory;
        try {
            factory = Class.forName(name).newInstance();
        } catch (ClassNotFoundException ex) {
            throw new BridgeException(ex);
        } catch (InstantiationException ex) {
            throw new BridgeException(ex);
        } catch (IllegalAccessException ex) {
            throw new BridgeException(ex);
        }
        if (!(factory instanceof XmlRpcTransportFactory)) {
            throw new BridgeException(
                "Invalid type " + factory.getClass().getName()
            );
        }
        this.client().setTransportFactory((XmlRpcTransportFactory) factory);
    }

    /**
     * Make an XML RPC call.
     * @param method The name of the remote method to call
     * @param params Paramter(s) to pass
     * @return The result
     * @throws BridgeException If something goes wrong
     */
    public Object call(final String method, final Object... params)
        throws BridgeException {
        Object response;
        try {
            response = this.client().execute(method, params);
        } catch (org.apache.xmlrpc.XmlRpcException ex) {
            throw new BridgeException(
                String.format(
                    "%s(%s) failed at %s",
                    method,
                    new Textualizer().text(params),
                    this.config.getServerURL().toString()
                ),
                ex
            );
        }
        Logger.debug(
            this,
            "#call(%s, %s): returned '%s'",
            method,
            new Textualizer().text(params),
            new Textualizer().text(response)
        );
        return response;
    }

    /**
     * Make and return XML RPC client.
     * @return The client
     */
    private XmlRpcClient client() {
        synchronized (this) {
            if (this.client == null) {
                this.client = new XmlRpcClient();
                this.client.setConfig(this.config);
                this.client.setTransportFactory(
                    new XmlRpcCommonsProxyAwareTransportFactory(this.client)
                );
            }
            return this.client;
        }
    }

}
