/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.utils;

// maps and collections
import java.util.Collection;
import java.util.Map;

// for string manipulations
import org.apache.commons.lang.StringEscapeUtils;

/**
 * Textualizer of an object.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Textualizer.java 1350 2011-05-08 03:37:27Z yegor256@yahoo.com $
 */
public final class Textualizer {

    /**
     * Maximum length of the string to render.
     */
    private static final Integer MAX_STRING_LENGTH = 30;

    /**
     * Tail to attach to string, if it's too long.
     */
    private static final String TAIL = "...";

    /**
     * Comma between elements.
     */
    private static final String COMMA = ", ";

    /**
     * Open bracket.
     */
    private static final String BRACKET_OPEN = "[";

    /**
     * Close bracket.
     */
    private static final String BRACKET_CLOSE = "]";

    /**
     * Textualize an object.
     * @param object The object
     * @return The text form of it
     */
    public String text(final Object object) {
        String text;
        if (object instanceof String) {
            text = this.textualize((String) object);
        } else if (object instanceof Integer) {
            text = object.toString();
        } else if (object instanceof Long) {
            text = object.toString();
        } else if (object == null) {
            text = "NULL";
        } else if (object.getClass().isArray()) {
            text = this.textualize((Object[]) object);
        } else if (object instanceof Collection) {
            text = this.textualize(((Collection) object).toArray());
        } else if (object instanceof Map) {
            text = this.textualize((Map) object);
        } else {
            text = "?";
        }
        return text;
    }

    /**
     * Textualize a String.
     * @param txt The string
     * @return The text form of it
     */
    private String textualize(final String txt) {
        String result = txt;
        if (txt.length() > this.MAX_STRING_LENGTH) {
            result = txt.substring(
                0,
                this.MAX_STRING_LENGTH - this.TAIL.length()
            ) + this.TAIL;
        }
        return StringEscapeUtils.escapeJava(result);
    }

    /**
     * Textualize an array.
     * @param array The array
     * @return The text form of it
     */
    private String textualize(final Object[] array) {
        final StringBuffer buf = new StringBuffer();
        for (Object inner : array) {
            if (buf.length() > 0) {
                buf.append(this.COMMA);
            }
            buf.append(this.text(inner));
        }
        return this.BRACKET_OPEN + buf.toString() + this.BRACKET_CLOSE;
    }

    /**
     * Textualize a Map.
     * @param map The map
     * @return The text form of it
     */
    private String textualize(final Map<Object, Object> map) {
        final StringBuffer buf = new StringBuffer();
        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            if (buf.length() > 0) {
                buf.append(this.COMMA);
            }
            buf.append(entry.getKey())
                .append(":")
                .append(this.text(entry.getValue()));
        }
        return this.BRACKET_OPEN + buf.toString() + this.BRACKET_CLOSE;
    }

}
