/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.utils;

import com.ymock.util.RuntimeProblem;

// for manipulation of URLs
import java.net.URL;

// for manipulation of params
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

// apache http commons
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;

// for string manipulations
import org.apache.commons.lang.StringUtils;

/**
 * Builds one URL.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: URLBuilder.java 1614 2011-07-10 19:05:43Z guard $
 */
public final class URLBuilder {

    /**
     * Slash in path.
     */
    private static final String SLASH = "/";

    /**
     * Path.
     */
    private String path;

    /**
     * Anchor.
     */
    private String anchor;

    /**
     * Arguments.
     */
    private final Map<String, String> args =
        new HashMap<String, String>();

    /**
     * Public ctor.
     * @param pth URL path
     */
    public URLBuilder(final String pth) {
        this.path = StringUtils.strip(pth, this.SLASH);
    }

    /**
     * Add new path segment.
     * @param pth The path to append
     * @return this
     */
    public URLBuilder append(final String pth) {
        this.path += this.SLASH + StringUtils.strip(pth, this.SLASH);
        return this;
    }

    /**
     * Add new argument.
     * @param name The name
     * @param value The value
     * @return this
     */
    public URLBuilder put(final String name, final String value) {
        this.args.put(name, value);
        return this;
    }

    /**
     * Add anchor.
     * @param txt The anchor
     * @return this
     */
    public URLBuilder anchor(final String txt) {
        this.anchor = txt;
        return this;
    }

    /**
     * Public ctor.
     * @return The URL created
     */
    public URL build() {
        final HttpMethod method = new GetMethod(this.path);
        final List<NameValuePair> pairs = new ArrayList<NameValuePair>();
        for (Map.Entry<String, String> entry : this.args.entrySet()) {
            pairs.add(new NameValuePair(entry.getKey(), entry.getValue()));
        }
        method.setQueryString(pairs.toArray(new NameValuePair[pairs.size()]));
        String url;
        try {
            url = method.getURI().getEscapedURI();
        } catch (org.apache.commons.httpclient.URIException ex) {
            throw RuntimeProblem.make(ex);
        }
        if (this.anchor != null) {
            url += "#" + this.anchor;
        }
        try {
            return new URL(url);
        } catch (NumberFormatException ex) {
            throw RuntimeProblem.make(ex);
        } catch (java.net.MalformedURLException ex) {
            throw RuntimeProblem.make(ex);
        }
    }

}
